from fastapi import APIRouter
from pydantic import BaseModel
import azure.cosmos.exceptions as exceptions
from ..database.mycosmosdb import callConnect
import datetime
import logging


class CompanyMeasure(BaseModel):
    oid: str
    email: str
    metric: str
    s1: float
    s2: float
    s3: float
    s4: str
    s5: str
    s6: str
    s7: str
class CMets(BaseModel):
    oid: str
    email: str
class ISMeasure(BaseModel):
    oid: str
    email: str
    metric: str
    funding: str
    emissionsIntensity: bool
    emissionsIntensityFin: bool
    measureBaseline: bool
    measureBaselineFin: bool
    pickOffice: bool
    pickOfficeFin: bool
    biasedLang: bool
    biasedLangFin: bool
    createBoard: bool
    createBoardFin: bool
    diverseBoard: bool
    diverseBoardFin: bool
    scope3: bool
    scope3Fin: bool
    neutralTarget: bool
    neutralTargetFin: bool
    physicalProducts: bool
    physicalProductsFin: bool
    suppliers: bool
    suppliersFin: bool
    officeSupplies: bool
    officeSuppliesFin: bool
    officeRecycle: bool
    officeRecycleFin: bool
    productImpact: bool
    productImpactFin: bool
    carbonEmployee: bool
    carbonEmployeeFin: bool
    energyPlan: bool
    energyPlanFin: bool
    waterPlan: bool
    waterPlanFin: bool
    wastePlan: bool
    wastePlanFin: bool
    candidateScreenings: bool
    candidateScreeningsFin: bool
    humanResources: bool
    humanResourcesFin: bool
    unconsciousBias: bool
    unconsciousBiasFin: bool
    compensationAudit: bool
    compensationAuditFin: bool
    equitableCompensation: bool
    equitableCompensationFin: bool
    # supplierDiversity: bool
    # supplierDiversityFin: bool
    diversityInclusion: bool
    diversityInclusionFin: bool
    recruitmentProcess: bool
    recruitmentProcessFin: bool
    # externalMarketing: bool
    # externalMarketingFin: bool
    biasEmployees: bool
    biasEmployeesFin: bool
    regularEmployee: bool
    regularEmployeeFin: bool
    anonymousHotline: bool
    anonymousHotlineFin: bool
    socialImpact: bool
    socialImpactFin: bool
    employeeAffinity: bool
    employeeAffinityFin: bool
    employeeWellbeing: bool
    employeeWellbeingFin: bool
    employeeCivic: bool
    employeeCivicFin: bool
    formalDiscrimination: bool
    formalDiscriminationFin: bool
    formalMental: bool
    formalMentalFin: bool
    privacyRegulation: bool
    privacyRegulationFin: bool
    addingIndependent: bool
    addingIndependentFin: bool
    # financialCapabilities: bool
    # financialCapabilitiesFin: bool
    publishDiversity: bool
    publishDiversityFin: bool
    # trainEmployees: bool
    # trainEmployeesFin: bool
    regularEnterprise: bool
    regularEnterpriseFin: bool
    designateIndependent: bool
    designateIndependentFin: bool
    publicDisclosure: bool
    publicDisclosureFin: bool
    # corporateSustainability: bool
    # corporateSustainabilityFin: bool

class Company(BaseModel):
    oid: str
    city: str
    company: str

class ReqsNotifications(BaseModel):
    id: str
    email: str

class Reqs(BaseModel):
    id: str
class ReqsDelete(BaseModel):
    oid: str
    id: str
    email: str
class InviteCompany(BaseModel):
    oid: str
    city: str
    title: str
    fund: str
    email: str
    companyemail: str
    companyname: str



def read_item(container, doc_id, account_number):
    response = container.read_item(item=doc_id, partition_key=account_number)
    return response

def read_items(container):
    response = container.read_all_items()
    return response

def delete_item(container, doc_id, account_number):
    response = container.delete_item(item=doc_id, partition_key=account_number)
    return response

companyroute = APIRouter()



"""
Endpoint used to update the company profile.
"""
@companyroute.post('/companyprofile')
async def company_profile(dmet: CompanyMeasure):
    try:
        db_result = read_item(callConnect.container11, dmet.oid, dmet.email)
    except exceptions.CosmosHttpResponseError as e:
        logging.error('This is an error message for /companyprofile')
        return {"url": "/", "atype": False}


    if "profile" in db_result:
        if dmet.metric == "CompanyProfile":
            user = {
                "id": db_result["id"],
                "email": db_result["email"],
                "profile": []
            }
            mets = [z for z in db_result["profile"] if z["label"] != "CompanyProfile"]
            if ([z for z in db_result["profile"] if z["label"] == "CompanyProfile"]):
                cmeas = [{
                    "companyName": dmet.s5,
                    "industryName": dmet.s6,
                    "annualRevenue": dmet.s1,
                    "fundingStage": dmet.s7,
                    "fullTime": dmet.s2,
                    "partTime": dmet.s3
                }]
                newMet = {
                    "label": "CompanyProfile",
                    "metrics": [
                        {
                            "label": "1",
                            "value": cmeas
                        }
                    ]
                }
                mets.append(newMet)
                target = {
                    "profile": mets
                }
                user.update(target)
                callConnect.container11.upsert_item(body=user)

                oldName = db_result["profile"][0]["metrics"][0]["value"][0]["companyName"]
                oldEmail = db_result["email"]
                bb = {
                    "id": dmet.s5,
                    "industry": dmet.s6
                }
                
                try:
                    allowed_result = read_item(callConnect.container3, oldName, oldEmail)
                    if "approved" in allowed_result:
                        for ap in allowed_result["approved"]:
                            try:
                                user_result = read_item(callConnect.container, ap["id"], ap["city"])
                                mn = []
                                for f in user_result["funds"]:
                                    nota = [c for c in f["companies"] if c["id"] != oldName]
                                    if ([c for c in f["companies"] if c["id"] == oldName]):
                                        a = [c for c in f["companies"] if c["id"] == oldName][0]
                                        a.update(bb)
                                        nota.append(a)

                                    comps = {
                                        "companies": nota
                                    }
                                    f.update(comps)
                                    mn.append(f)
                                xc = {
                                    "funds": mn
                                }
                                user_result.update(xc)
                                callConnect.container.upsert_item(body=user_result)
                            except exceptions.CosmosHttpResponseError as e:
                                pass
                        
                    a = {
                        "id": dmet.s5
                    }
                    delete_item(callConnect.container3, oldName, oldEmail)
                    allowed_result.update(a)
                    callConnect.container3.upsert_item(body=allowed_result)

                except exceptions.CosmosHttpResponseError as e:
                    logging.error('This is an error message for /companyprofile in allowed container')

                try:
                    companies_result = read_item(callConnect.container2, oldName, oldEmail)
                    newC = {
                        "id": dmet.s5,
                        "industry": dmet.s6
                    }
                    delete_item(callConnect.container2, oldName, oldEmail)
                    companies_result.update(newC)
                    callConnect.container2.upsert_item(body=companies_result)

                except exceptions.CosmosHttpResponseError as e:
                    logging.error('This is an error message for /companyprofile in companies container')

                try:
                    pass_result = read_item(callConnect.container5, dmet.oid, dmet.oid)
                    bagPass = [a for a in pass_result["members"]]
                    for ps in bagPass:
                        try:
                            shared_result = read_item(callConnect.container13, ps["name"], ps["name"])
                            if ([t for t in shared_result["team"] if t["oid"] == dmet.oid]):
                                allSR = [b for b in shared_result["team"] if b["oid"] != dmet.oid]
                                bagSR = [v for v in shared_result["team"] if v["oid"] == dmet.oid][0]
                                newSR = {
                                    "cname": dmet.s5
                                }
                                bagSR.update(newSR)
                                allSR.append(bagSR)
                                teamSR = {
                                    "team": allSR
                                }
                                shared_result.update(teamSR)
                                callConnect.container13.upsert_item(body=shared_result)
                        except exceptions.CosmosHttpResponseError as e:
                            logging.error('This is an error message for /companyprofile in sharedpass container')
                except exceptions.CosmosHttpResponseError as e:
                    logging.error('This is an error message for /companyprofile in sharedpass container')

                
                try:
                    user_record = read_item(callConnect.container, dmet.oid, dmet.s4)
                    if user_record["plan_id"] == "price_1KU9FKJnvGQfuwqXeh1Sikvd" or user_record["plan_id"] == "price_1KU9EIJnvGQfuwqX1pAkeLP0" or user_record["plan_id"] == "price_1MC79wJnvGQfuwqXeLIJXVws" or user_record["plan_id"] == "price_1MN2vNJnvGQfuwqX3ZT4NiyE":
                        for f in user_record["funds"]:
                            bagCs = [co for co in f["companies"]]
                            for cn in bagCs:
                                try:
                                    new_allowed_record = read_item(callConnect.container3, cn["id"], cn["email"])
                                    if "approved" in new_allowed_record:
                                        all_approved = [pp for pp in new_allowed_record["approved"] if pp["name"] != oldName]
                                        nName = {
                                            "name": dmet.s5
                                        }
                                        current_approved = [pp for pp in new_allowed_record["approved"] if pp["name"] == oldName]
                                        for ca in current_approved:
                                            ca.update(nName)
                                            all_approved.append(ca)
                                        ap = {
                                            "approved": all_approved
                                        }
                                        new_allowed_record.update(ap)
                                        callConnect.container3.upsert_item(body=new_allowed_record)

                                except exceptions.CosmosHttpResponseError as e:
                                    logging.error('This is an error message for /companyprofile in allowed container')
                                pass
                except exceptions.CosmosHttpResponseError as e:
                    logging.error('This is an error message for /companyprofile in sharedpass container')


            else:
                user = {
                        "id": db_result["id"],
                        "email": db_result["email"],
                        "profile": []
                }
                mets = [z for z in db_result["profile"] if z["label"] != "CompanyProfile"]
                newMet = {
                    "label": "CompanyProfile",
                    "metrics": [
                        {
                            "label": "1",
                            "value": [
                                {
                                    "companyName": dmet.s5,
                                    "industryName": dmet.s6,
                                    "annualRevenue": dmet.s1,
                                    "fundingStage": dmet.s7,
                                    "fullTime": dmet.s2,
                                    "partTime": dmet.s3
                                }
                            ]
                        }
                    ]
                }
                mets.append(newMet)
                target = {
                    "profile": mets
                }
                newCompany = {
                    "id": dmet.s5,
                    "email": db_result["email"],
                    "requested": []
                }
                callConnect.container3.upsert_item(body=newCompany)
                newCom = {
                    "id": dmet.s5,
                    "email": db_result["email"],
                    "industry": dmet.s6,
                    "oid": dmet.oid
                }
                callConnect.container2.upsert_item(body=newCom)
                user.update(target)
                callConnect.container11.upsert_item(body=user)



@companyroute.post('/getcompanyprofile')
async def get_company_profile(dmet: CMets):
    try:
        db_result = read_item(callConnect.container11, dmet.oid, dmet.email)
    except exceptions.CosmosHttpResponseError as e:
        logging.error('This is an error message for /getcompanyprofile')
        return {"url": "/", "atype": False}
    

    if db_result:
        if "profile" in db_result:
            return {"pmets": db_result["profile"]}
    else:
        pass

"""
Endpoint used to grab all of the improvement strategies
"""
@companyroute.post('/getismeasure')
async def get_is_measure(isme: CMets):
    try:
        db_result = read_item(callConnect.container12, isme.oid, isme.email)
    except exceptions.CosmosHttpResponseError as e:
        logging.error('This is an error message for /getismeasure')
        return {"url": "/", "atype": False}
    

    if "improvement" in db_result:
        return {"isme": db_result["improvement"]}


@companyroute.post('/ismeasure')
async def is_measure(isme: ISMeasure):
    try:
        db_result = read_item(callConnect.container12, isme.oid, isme.email)

    except exceptions.CosmosHttpResponseError as e:
        logging.error('This is an error message for /ismeasure')
        return {"url": "/", "atype": False}
    
    
    if "improvement" in db_result:
        if isme.metric == "IS":
            user = {
                "id": db_result["id"],
                "email": db_result["email"],
                "improvement": []
            }
            nowZ = datetime.datetime.now()
            yearZ = nowZ.strftime("%Y")
            monthZ = nowZ.strftime("%m")
            dayZ = nowZ.strftime("%d")
            all_timeZ = yearZ+"-"+monthZ+"-"+dayZ
            
            tmeas = []
            tmeas.append({
                    "datestr": all_timeZ,
                    "funding": isme.funding,
                    "emissionsIntensity": isme.emissionsIntensity,
                    "emissionsIntensityFin": isme.emissionsIntensityFin,
                    "measureBaseline": isme.measureBaseline,
                    "measureBaselineFin": isme.measureBaselineFin,
                    "pickOffice": isme.pickOffice,
                    "pickOfficeFin": isme.pickOfficeFin,
                    "biasedLang": isme.biasedLang,
                    "biasedLangFin": isme.biasedLangFin,
                    "createBoard": isme.createBoard,
                    "createBoardFin": isme.createBoardFin,
                    "diverseBoard": isme.diverseBoard,
                    "diverseBoardFin": isme.diverseBoardFin,
                    "scope3": isme.scope3,
                    "scope3Fin": isme.scope3Fin,
                    "neutralTarget": isme.neutralTarget,
                    "neutralTargetFin": isme.neutralTargetFin,
                    "physicalProducts": isme.physicalProducts,
                    "physicalProductsFin": isme.physicalProductsFin,
                    "suppliers": isme.suppliers,
                    "suppliersFin": isme.suppliersFin,
                    "officeSupplies": isme.officeSupplies,
                    "officeSuppliesFin": isme.officeSuppliesFin,
                    "officeRecycle": isme.officeRecycle,
                    "officeRecycleFin": isme.officeRecycleFin,
                    "productImpact": isme.productImpact,
                    "productImpactFin": isme.productImpactFin,
                    "carbonEmployee": isme.carbonEmployee,
                    "carbonEmployeeFin": isme.carbonEmployeeFin,
                    "energyPlan": isme.energyPlan,
                    "energyPlanFin": isme.energyPlanFin,
                    "waterPlan": isme.waterPlan,
                    "waterPlanFin": isme.waterPlanFin,
                    "wastePlan": isme.wastePlan,
                    "wastePlanFin": isme.wastePlanFin,
                    "candidateScreenings": isme.candidateScreenings,
                    "candidateScreeningsFin": isme.candidateScreeningsFin,
                    "humanResources": isme.humanResources,
                    "humanResourcesFin": isme.humanResourcesFin,
                    "unconsciousBias": isme.unconsciousBias,
                    "unconsciousBiasFin": isme.unconsciousBiasFin,
                    "compensationAudit": isme.compensationAudit,
                    "compensationAuditFin": isme.compensationAuditFin,
                    "equitableCompensation": isme.equitableCompensation,
                    "equitableCompensationFin": isme.equitableCompensationFin,
                    # "supplierDiversity": isme.supplierDiversity,
                    # "supplierDiversityFin": isme.supplierDiversityFin,
                    "diversityInclusion": isme.diversityInclusion,
                    "diversityInclusionFin": isme.diversityInclusionFin,
                    "recruitmentProcess": isme.recruitmentProcess,
                    "recruitmentProcessFin": isme.recruitmentProcessFin,
                    # "externalMarketing": isme.externalMarketing,
                    # "externalMarketingFin": isme.externalMarketingFin,
                    "biasEmployees": isme.biasEmployees,
                    "biasEmployeesFin": isme.biasEmployeesFin,
                    "regularEmployee": isme.regularEmployee,
                    "regularEmployeeFin": isme.regularEmployeeFin,
                    "anonymousHotline": isme.anonymousHotline,
                    "anonymousHotlineFin": isme.anonymousHotlineFin,
                    "socialImpact": isme.socialImpact,
                    "socialImpactFin": isme.socialImpactFin,
                    "employeeAffinity": isme.employeeAffinity,
                    "employeeAffinityFin": isme.employeeAffinityFin,
                    "employeeWellbeing": isme.employeeWellbeing,
                    "employeeWellbeingFin": isme.employeeWellbeingFin,
                    "employeeCivic": isme.employeeCivic,
                    "employeeCivicFin": isme.employeeCivicFin,
                    "formalDiscrimination": isme.formalDiscrimination,
                    "formalDiscriminationFin": isme.formalDiscriminationFin,
                    "formalMental": isme.formalMental,
                    "formalMentalFin": isme.formalMentalFin,
                    "privacyRegulation": isme.privacyRegulation,
                    "privacyRegulationFin": isme.privacyRegulationFin,
                    "addingIndependent": isme.addingIndependent,
                    "addingIndependentFin": isme.addingIndependentFin,
                    # "financialCapabilities": isme.financialCapabilities,
                    # "financialCapabilitiesFin": isme.financialCapabilitiesFin,
                    "publishDiversity": isme.publishDiversity,
                    "publishDiversityFin": isme.publishDiversityFin,
                    # "trainEmployees": isme.trainEmployees,
                    # "trainEmployeesFin": isme.trainEmployeesFin,
                    "regularEnterprise": isme.regularEnterprise,
                    "regularEnterpriseFin": isme.regularEnterpriseFin,
                    "designateIndependent": isme.designateIndependent,
                    "designateIndependentFin": isme.designateIndependentFin,
                    "publicDisclosure": isme.publicDisclosure,
                    "publicDisclosureFin": isme.publicDisclosureFin
                    # "corporateSustainability": isme.corporateSustainability,
                    # "corporateSustainabilityFin": isme.corporateSustainabilityFin
                })
            newMet = [{
                "label": "IS",
                "metrics": tmeas
            }]
            target = {
                "improvement": newMet
            }
            user.update(target)
            callConnect.container12.upsert_item(body=user)
        

"""
Endpoint used to grab the funds that a user has. !! make this appear in the UI and we have made progress, then we will post req that an investor wants to access data. read that post and display a popup to see if that company wants to give the investors access to the data. If access is granted that is another data category where we remove the request for data and we post the investors email into the companies allowed emails to read that companies data.
"""
@companyroute.post('/getcompanies')
def grab_companies():
    company_list = []
    try:
        db_result = read_items(callConnect.container2)

        if db_result:
            for doc in list(db_result):
                docx = {
                    "title": doc["id"],
                    "email": doc["email"]
                }
                company_list.append(docx)

            return {"companies": company_list}
        else:
            logging.error('There are no companies')
        

    except exceptions.CosmosHttpResponseError as e:
        logging.error('There are no companies')
        return {"url": "/", "atype": False}



"""
Endpoint used to grab the open data requests that a user has
"""
@companyroute.post('/getrequests')
def get_requests(reqs: ReqsNotifications):
    try:
        db_result = read_item(callConnect.container3, reqs.id, reqs.email)        

    except exceptions.CosmosHttpResponseError as e:
        logging.error('There is an error in /getrequests')
        return {"url": "/", "atype": False}

    if db_result:
        if "requested" in db_result:
            return {"reqs": db_result["requested"]}

"""
Endpoint used to grab the approved data requests that a user has
"""
@companyroute.post('/getapproved')
async def get_approved(reqs: ReqsNotifications):
    try:
        db_result = read_item(callConnect.container3, reqs.id, reqs.email)        

    except exceptions.CosmosHttpResponseError as e:
        logging.error('There is an error in /getapproved')
        return {"url": "/", "atype": False}

    if db_result:
        if "approved" in db_result:
            return {"approved": db_result["approved"]}


"""
Endpoint used to grab the team users.
"""
@companyroute.post('/getmembers')
def get_members(reqs: Reqs):
    try:
        db_result = read_item(callConnect.container5, reqs.id, reqs.id)
        
    except exceptions.CosmosHttpResponseError as e:
        logging.error('There is an error in /getmembers')
        return {"url": "/", "atype": False}


    if "members" in db_result:
        return {"members": db_result["members"]}
    else:
        pass


"""
Endpoint used to grab the team access.
"""
@companyroute.post('/getaccess')
def get_access(reqs: Reqs):
    try:
        db_result = read_item(callConnect.container13, reqs.id, reqs.id)
        
    except exceptions.CosmosHttpResponseError as e:
        logging.error('There is an error in /getmembers')
        pass


    if "team" in db_result:
        return {"team": db_result["team"]}
    else:
        pass

"""
Endpoint used to delete access.
"""
@companyroute.post('/deleteaccess')
async def delete_access(reqs: ReqsDelete):
    try:
        delete_item(callConnect.container5, reqs.id, reqs.id)
        try:
            find_emails = read_item(callConnect.container5, reqs.oid, reqs.oid)
            if "members" in find_emails: 
                if ([d for d in find_emails["members"] if d["name"] == reqs.id]):
                    emails = [d for d in find_emails["members"] if d["name"] != reqs.id]
                    allowed = {
                        "id": reqs.oid,
                        "members": emails
                    }
                    callConnect.container5.upsert_item(body=allowed)
            else:
                pass
        
        except exceptions.CosmosHttpResponseError as e:
            pass
            
        
    except exceptions.CosmosHttpResponseError as e:
        try:
            db_result2 = read_item(callConnect.container13, reqs.email, reqs.email)
            if "team" in db_result2:
                if ([d for d in db_result2["team"] if d["email"] == reqs.id]):
                    a = [d for d in db_result2["team"] if d["email"] != reqs.id]
                    allowed = {
                        "id": reqs.email,
                        "oid": reqs.oid,
                        "team": a
                    }
                    callConnect.container13.upsert_item(body=allowed)
            else:
                pass
            try:
                find_emails = read_item(callConnect.container5, reqs.oid, reqs.oid)
                if "members" in find_emails:
                    if ([d for d in find_emails["members"] if d["name"] == reqs.id]):
                        emails = [d for d in find_emails["members"] if d["name"] != reqs.id]
                        allowed = {
                            "id": reqs.oid,
                            "members": emails
                        }
                        callConnect.container5.upsert_item(body=allowed)
                else:
                    pass
        
        
            except exceptions.CosmosHttpResponseError as e:
                pass
        
        
        except exceptions.CosmosHttpResponseError as e:
            pass



"""
Endpoint used to generate a SendGrid email to send to the company.
"""
@companyroute.post('/filldatarequest')
async def fullfill_data_request(iCompany: InviteCompany):
    try:
        db_result3 = read_item(callConnect.container3, iCompany.companyname, iCompany.companyemail)

    except exceptions.CosmosHttpResponseError as e:
        logging.error('There is an error in /filldatarequest container3')
        return {"url": "/", "atype": False}

    if "requested" in db_result3:
        allpens = [pen for pen in db_result3["requested"] if pen["id"] != iCompany.oid]        
        
        if "approved" in db_result3:
            approvedfunds = []
            if ([d for d in db_result3["approved"] if d["id"] == iCompany.oid]):
                approvedfunds = [d for d in db_result3["approved"] if d["id"] == iCompany.oid][0]["funds"]
            approvedlist = db_result3["approved"]
            approvedfunds.append({"name": iCompany.fund})
            allowedpennew = {
                "id": iCompany.oid,
                "city": iCompany.city,
                "name": iCompany.title,
                "email": iCompany.email,
                "funds": approvedfunds
            }
            approvedlist.append(allowedpennew)
            allowed = {
                "id": db_result3["id"],
                "email": db_result3["email"],
                "requested": allpens,
                "approved": approvedlist
            }
        else:
            approvedfunds = []
            approvedfunds.append({"name": iCompany.fund})
            allowedpennew = {
                "id": iCompany.oid,
                "city": iCompany.city,
                "name": iCompany.title,
                "email": iCompany.email,
                "funds": approvedfunds
            }
            allowed = {
                "id": db_result3["id"],
                "email": db_result3["email"],
                "requested": allpens,
                "approved": [allowedpennew]
            }
        callConnect.container3.upsert_item(body=allowed)
        

    try:
        db_result1 = read_item(callConnect.container2, iCompany.companyname, iCompany.companyemail)

    except exceptions.CosmosHttpResponseError as e:
        logging.error('There is an error in /filldatarequest container2')
        return {"url": "/", "atype": False}


    try:
        db_result = read_item(callConnect.container, iCompany.oid, iCompany.city)

    except exceptions.CosmosHttpResponseError as e:
        logging.error('There is an error in /filldatarequest container')
        return {"url": "/", "atype": False}

    if "funds" in db_result:
        user2 = {
            "id": db_result["id"],
            "city": db_result["city"],
            "email": db_result["email"],
            "session_id": db_result["session_id"],
            "sub_created": db_result["sub_created"],
            "plan_id": db_result["plan_id"],
            "funds": db_result["funds"],
            "subs": db_result["subs"]
        }
        allfunds = [d for d in db_result["funds"] if d["fund"] != iCompany.fund]
        replacefund = [d for d in db_result["funds"] if d["fund"] == iCompany.fund][0]
        if "pending" in replacefund:
            ccc = replacefund["companies"]
            ccc.append({"id": db_result1["id"], "email": db_result1["email"], "industry": db_result1["industry"], "oid": db_result1["oid"]})
            ppp = [p for p in replacefund["pending"] if p["name"] != iCompany.companyname]
            addfund = {
                "fund": replacefund["fund"],
                "fundstartdate": replacefund["fundstartdate"],
                "companies": ccc,
                "pending": ppp
            }
        allfunds.append(addfund)
        fund = {
        "funds": allfunds
        }
        user2.update(fund)
        callConnect.container.upsert_item(body=user2)


